package seniorproj;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.StringTokenizer;

/**
 * Class used to open saved Warehouse states and save current states. 
 * It filters for .txt files and when opening and saves to your
 * current working directory when saving. 
 * 
 * @author Steven Weigand
 *
 */
public class ChooseFile {

	/**
	 * Constructor used to initialize the filter for opening saved
	 * Warehouse states.
	 */
	public ChooseFile() {
		filter();
	}

	/**
	 * Creates FileChooser and FileFilter for filtering .txt files. It allows
	 * the user to be able to select .txt files from their current working
	 * directory. 
	 */
	private void filter() {

		JFileChooser fc = new JFileChooser();

		FileFilter filter = new FileFilter()
		{

			/**
			 * Checks extensions of files to see if they match .txt when
			 * displaying the files that are compatible for opening.
			 * 
			 * @param f 	the file having its extension compared
			 * @return		boolean indicating if the file was accepted or not
			 */
			public boolean accept(File f) {

				if (f.isDirectory()) { 
					return true;
				}

				if (!f.isFile()) {
					return false;
				}

				String ext = getExtension(f);

				if (ext.equals("txt")) {
					return true;
				}
				return false;
			}

			/**
			 * Provides a description of what files are being filtered for.
			 * 
			 * @return		String containing file descriptions
			 */
			public String getDescription() {
				return ".txt Files";
			}

			/**
			 * Retrieves the extension of a given File for comparison.
			 * 
			 * @param f		the File where the extension will be extracted from
			 * @return		String containing the extension
			 */
			public String getExtension(File f) 
			{
				String filename = f.getName();

				int len = filename.lastIndexOf('.');

				if (len <= 0 || len >= filename.length() -1) {
					return "";
				}
				else return filename.substring(len+1).toLowerCase();
			}
		} ; 
		
		//Add filter
		fc.addChoosableFileFilter(filter);
        
        //Set to current working directory
		fc.setCurrentDirectory(new File("."));
		
		//Show FileChooser
		int result = fc.showDialog(null, "Open");

		//If open was successful let user know
        if (result == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
          
            System.out.println("Opening: " + file.getName() + ".");
            System.out.println(file.getPath());
        }

        else {
            System.out.println("Open command cancelled by user.");
        }
	}
	
	/**
	 * Parses through a text file containing a saved state of a data warehouse.
	 * When done parsing a warehouse will appear where it left off.
	 * 
	 * @param name		Name of the file to be opened
	 * @param path		The path where the file can be found
	 */
	public static void openFile(String name, String path) {
		
   		/*try {
   			BufferedReader in = new BufferedReader(new FileReader("WarehouseState.txt"));
   			StringTokenizer tok = null;
   			String line = null;
   			int setNum = 0;
   			int tables = 0;
   			String code = null;
   			String setNames[];
   			String tabNames[];
   			
   			line = in.readLine();
   			
   			//Get Dataset num/names
   			tok = new StringTokenizer(line);
   			tok.nextToken();
   			setNum = Integer.parseInt(tok.nextToken());
   			setNames = new String[setNum];
   			
   			for(int i = 0; i < setNum; i++) {
   				setNames[i] = in.readLine();
   				System.out.println("Set Names: "+ setNames[i]);
   			}
   			
   			//Get table num/names
   			line = in.readLine();
   			tok = new StringTokenizer(line);
   			tok.nextToken();
   			tables = Integer.parseInt(tok.nextToken());
   			tabNames = new String[tables];
   			
   			for(int i = 0; i < tables; i++) {
   				tabNames[i] = in.readLine();
   				System.out.println("Table Names: "+ tabNames[i]);
   			}
   			
   			//Blank line
   			in.readLine();
   			
   			//Start parsing datasets
   			for(int i = 0; i < setNum; i++) {
   				String sName = setNames[i]; 
   				
   				//Dataset Name
   				in.readLine();
   	   			//Tables keyword
   				line = in.readLine();
   	   			   	
   				
   				tok = new StringTokenizer(line);
   				tok.nextToken();
   	   			tables = Integer.parseInt(tok.nextToken());
   	   			
   	   			for(int j = 0; j < tables; j++) {
   	   				line = in.readLine();
   	   				
   	   				
   	   			}
   			}
   			
		} catch (IOException e1) {
			e1.printStackTrace();
		}*/
	}
	
	/**
	 * Saves the current state of the warehouse in a .txt file named
	 * WarehouseState.txt in your current working directory.
	 */
	public static void saveState() {
		
		PrintWriter out = null;
		DataSet temp = null;
		Object[] tables = null;
		
		//Open printwriter for writing
   		try {
   			out = new PrintWriter(new BufferedWriter(new FileWriter("WarehouseState.txt")));			
			
		} catch (IOException e1) {
			e1.printStackTrace();
		}		
		
		//Write each dataset to file
		out.println("Datasets: "+CreateSet.setsIndex);
		for(int i = 0; i < CreateSet.setsIndex; i++) {
			temp = CreateSet.sets[i];
			out.println(temp.getName());
			
			out.println("Tables: "+temp.getNames().length);
			tables = temp.getNames();

			for(int j = 0; j < tables.length; j++) {
				if(tables[j] != null) {
					out.println(tables[j]);
				}
			}
		}
		
		//Write columns in datasets to file
		String view = null;
        for(int i = 0; i < CreateSet.setsIndex; i++) {
        	
			out.println();
	        view = "Dataset "+CreateSet.sets[i].getName()+"\n";
	        view += "     Tables: "+CreateSet.sets[i].getNames().length+"\n";
	        
	        String[] tabs = new String[CreateSet.sets[i].getNames().length];
	        tabs = (String[]) CreateSet.sets[i].getNames();
	        
	        for(int j = 0; j < tabs.length; j++) {
	        	if(tabs[j] != null) {
	        		view += "\t"+tabs[j]+"\n";
	
		        	Column[] tcolumns = DBConnect.getColumns(tabs[j]);
		        	view += "\n\tColumns:\n";
		        	for(int k = 0; k < tcolumns.length; k++) {
		        		view += "\t\t"+tcolumns[k].getName()+"\n";
		        	}
		        	view += "\n";
	        	}
	        }
        }
	        
        //Print overview of warehouse to file
		if(TableAnalysis.warehouse) {

	        view += "\nCurrent Warehouse\n";
	        view += "     Tables: \n";
	        for(int i = 0; i < DBConnect.tindex; i++) {
	        	view += "\t"+DBConnect.refNames[i]+"\n";
	        	
	        	Column[] cols = DBConnect.getColumns(DBConnect.refNames[i]);
				view += "\n\tColumns:\n";
	        	for(int j = 0; j < cols.length; j++) {
	        		if(cols[j] != null) {
	        			view += "\t\t"+cols[j].getName()+"\n";
	        		}
	        	}
	        }
	        
	        view += "\t"+SelectAttr.tablename.toUpperCase()+"\n";
	        view += "\n\tColumns:\n";
			for(int i = 0; i < DBConnect.colnum; i++) {
				if(DBConnect.used[i] != null) {
					view += "\t\t"+DBConnect.used[i].getName()+"\n";
				}
			}
	    	
			for(int i = 0; i < SelectDependents.full.length; i++) {
				view += "\t\t"+SelectDependents.full[i]+"\n";
			}
			
			out.println(view);
			
			//Write sql code to file
			out.println(TableAnalysis.code);
		}
		
		out.println("EOF");

		out.close();
	}
}
