package seniorproj;

/**
 * Creates a column object containing information regarding the column name, the table its from, 
 * and the numerical attributes.
 * 
 * @author Steven Weigand
 *
 */
public class Column {
	
	String name;
	String tname;
	String type;
	int length = 0;
	int precision = 0;
	int scale = 0;
	
	/**
	 * Default Column constructor for initialization.
	 */
	public Column() {}
	
	/**
	 * Column constructor for initializing and creating a Column object.
	 * 
	 * @param name		Name of the column
	 * @param tname		Name of the column's table
	 * @param type		Type of the column
	 * @param length	The length of whatever type the column holds
	 * @param precision	The precision of whatever type the column holds
	 * @param scale		The scale of whatever type the column holds
	 */
	public Column(String name, String tname, String type, int length, int precision, int scale) {
		this.name = name;
		this.tname = tname;
		this.type = type;
		this.length = length;
		this.precision = precision;
		this.scale = scale;
	}
	
	/**
	 * Retrieve the Column's name.
	 * 
	 * @return		String containing column name
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Retrieve the Column's tablename.
	 * 
	 * @return		String containing column table name.
	 */
	public String getTname() {
		return tname;
	}
	
	/**
	 * Retrieve the Column's type.
	 * 
	 * @return		String containing column type.
	 */
	public String getType() {
		return type;
	}
	
	/**
	 * Retrieve the Column's length.
	 * 
	 * @return		int containing column length
	 */
	public int getLength() {
		return length;
	}
	
	/**
	 * Retrieve the Column's precision.
	 * 
	 * @return		int containing column precision
	 */
	public int getPrec() {
		return precision;
	}
	
	/**
	 * Retrieve the Column's scale.
	 * 
	 * @return		int containing column scale
	 */
	public int getScale() {
		return scale;
	}
}
