package seniorproj;

import javax.swing.*;  

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Creates a GUI for a user to sign in with his/her username and password
 * with all of the proper button action listeners.
 * 
 * @author Steven Weigand
 *
 */
public class ConnectionGUI {
	
	/**
	 * The frame containing the GUI components.
	 */
	static JFrame frame = null;
	
	/**
	 * The container specifying the GUI components and attributes.
	 */
	static Container content = null;
	
	/**
	 * The textfield where the user specifies a database.
	 */
	static JTextField db = null;
	
	/**
	 * The textfield where the user specifies a username.
	 */
	static JTextField username = null;
	
	/**
	 * The textfield where the user specifies a password.
	 */
	static JTextField input = null;
	
	/**
	 * The button where the user submits his/her info.
	 */
	static JButton submit = null;
	
	/**
	 * The button where the user cancels the signin.
	 */
	static JButton cancel = null;
	
	/**
	 * Creates GUI layout for a user to sign in to the database.
	 */
    public static void createConnectGUI() {
        //Create and set up the window.
        frame = new JFrame("CSC 467: Logon");
        
        content = frame.getContentPane();
        
        content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
        
        //Enter database name here
        JPanel datab = new JPanel();
        JLabel dlabel = new JLabel("Database:");
        datab.add(dlabel);
        
        db = new JTextField(10);
        datab.add(db);
        content.add(datab);
        
        //Enter username name here
        JPanel user = new JPanel();
        JLabel label = new JLabel("Username:");
        user.add(label);
        
        username = new JTextField(10);
        user.add(username);
        
        content.add(user);
        
        //Enter password here
        JPanel pass = new JPanel();
        JLabel plabel = new JLabel("Password:");
        pass.add(plabel);
        
        input = new JPasswordField(10);
        pass.add(input);    
        
        content.add(pass);
        
        //Create Submit and Cancel buttons
        JPanel buttons = new JPanel();
        submit = new JButton("Submit");
        submit.addActionListener(new ButtonListener());
        
        cancel = new JButton("Cancel");
        cancel.addActionListener(new ButtonListener2());
        
        buttons.add(submit);
        buttons.add(cancel);
        
        content.add(buttons);

        //Display the window.
        frame.setPreferredSize(new Dimension(350, 200));
        frame.setLocationRelativeTo(MainWindow.frame);
        frame.pack();
        frame.setVisible(true);
    }
    
    /**
     * The listener for "submit" button allowing a user to login when pressed.
     */
    static class ButtonListener implements ActionListener {

    	/**
    	 * Used to catch when the submit button is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		boolean correct;
    		
    		correct = DBConnect.connection(username.getText(), input.getText(), db.getText());
    		
    		if(correct) {
	    		MainWindow.loggedin = true;
	    		frame.setVisible(false);
	    		MainWindow.frame.setVisible(false);
	    		MainWindow.createMainWindow();
    		}
    		else {
    			JOptionPane.showMessageDialog(frame, "Invalid Username/Password");    				
    		}
       	}
    }
    
    /**
     * The listener for the "cancel" button allowing the user to cancel the login.
     */
    static class ButtonListener2 implements ActionListener {
       	/**
    	 * Used to catch when the cancel button is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		frame.setVisible(false);
       	}
    }
}
