package seniorproj;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Creates GUI for user to select what tables to place into a new 
 * data set. Once a data set is created it appears on the user's main
 * window.
 * 
 * @author Steven Weigand
 *
 */
public class CreateSet {
	
	/**
	 * The frame containing the GUI components.
	 */
	static JFrame frame = null;
	
   	/**
	 * Array of global datasets.
	 */
	static DataSet sets[] = new DataSet[100];
	
   	/**
	 * Index tracking the number of global datasets.
	 */
	static int setsIndex = 0;
	
  	/**
	 * ListModel for holding selected tables.
	 */
	static DefaultListModel listModel = null;
	
  	/**
	 * Textfield containing name of the dataset.
	 */
	static JTextField name = null;
	
  	/**
	 * List for holding selected tables.
	 */
	static JList dsTables = null;
	
  	/**
	 * Button for adding tables to the list.
	 */
	static JButton add = null;
	
  	/**
	 * Button for removing tables to the list.
	 */
	static JButton rem = null;
	
  	/**
	 * Button for creating the dataset.
	 */
	static JButton create = null;
	
  	/**
	 * Object containing currently selected value from left side.
	 */
	static Object selectedValue = null;
	
  	/**
	 * Object containing currently selected value from right side.
	 */
	static Object selectedValue2 = null;
	
  	/**
	 * Array containing available columns.
	 */
    static Column temp[] = null;

    
    /**
     * Creates the GUI layout for listing the available tables and allowing the 
     * user to create a dataset.
     */
	public static void createSetGUI() {
		
        frame = new JFrame("CSC 467: Create Dataset");

        Container content = frame.getContentPane();
        content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
        content.setBackground(Color.GRAY);
        
        JPanel expl = new JPanel();
        JLabel setExpl = new JLabel("Choose Tables For Dataset");
        expl.add(setExpl);
        content.add(expl);   
        
        JPanel display = new JPanel();
        display.setLayout(new BorderLayout());
        
        //Enter name 
        JPanel enterName = new JPanel();
        JLabel choose = new JLabel("Name:");
        enterName.add(choose); 
        name = new JTextField(15);
        enterName.add(name);
        
        //Add enter name
        display.add(enterName, BorderLayout.NORTH);
        
        //User tables to select from panel
        JPanel yourTables = new JPanel();
        
        String[] tnames = DBConnect.getTableNames();
        
        //Create tree view of available tables to choose from
        DefaultMutableTreeNode root = new DefaultMutableTreeNode("Tables");
        DefaultMutableTreeNode parent;
        DefaultMutableTreeNode child;
        
        for(int i = 0; i < tnames.length; i++) {
        	if(tnames[i] != null) {
        		parent = new DefaultMutableTreeNode(tnames[i]);
        		temp = DBConnect.getColumns(tnames[i]);
	    		for(int j = 0; j < DBConnect.columns.length; j++) {
	    			if(temp[j] != null) {
	    				child = new DefaultMutableTreeNode(DBConnect.columns[j].getName());
	    				parent.add(child);
	    			}
	    		}
	    		root.add(parent);
        	}
        }
        
        final JTree tree = new JTree(root);
        tree.setPreferredSize(new Dimension(200, 500));

        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

		//Listen for when the selection changes.
		tree.addTreeSelectionListener(new TreeSelectionListener() {
		    public void valueChanged(TreeSelectionEvent e) {
		        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
		                           tree.getLastSelectedPathComponent();
		        
		        if (node == null) return;
		        else {
		        	selectedValue = node.getUserObject();
		        }
		    }
		});
		
        JScrollPane pane = new JScrollPane(tree);
        pane.setPreferredSize(new Dimension(200, 500));
		
        yourTables.add(pane);
        display.add(yourTables, BorderLayout.WEST);
		
        
        //Add/Remove buttons panel
        JPanel buttons = new JPanel();
        JPanel buttons1 = new JPanel();
        buttons1.setPreferredSize(new Dimension(800, 50));
        JPanel buttons2 = new JPanel();
        buttons2.setPreferredSize(new Dimension(800, 50));
        JPanel padding = new JPanel();
        padding.setPreferredSize(new Dimension(800, 100));
        buttons.add(padding);
        
        //Create add/remove buttons
        add = new JButton("Add >>");
        add.addActionListener(new ButtonListener2());
        buttons1.add(add);
        rem = new JButton("<< Remove");
        rem.addActionListener(new ButtonListener3());
        buttons2.add(rem);
        
        buttons.add(buttons1);
        buttons.add(buttons2);
        display.add(buttons, BorderLayout.CENTER);
        
        //Added Tables panel of chosen tables
        JPanel setTables = new JPanel();
        listModel = new DefaultListModel();
        dsTables = new JList(listModel);
        dsTables.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        dsTables.addListSelectionListener(new ListSelectionListener() {
        	public void valueChanged(ListSelectionEvent evt) {
        		JList list = (JList)evt.getSource();
                selectedValue2 = list.getSelectedValue();
        	}
        });
        dsTables.setPreferredSize(new Dimension(200, 500));
        JScrollPane pane2 = new JScrollPane(dsTables);
        pane2.setPreferredSize(new Dimension(200, 500));

        setTables.add(pane2);

        display.add(setTables, BorderLayout.EAST);
        
        //Create button panel
        JPanel end = new JPanel();
        create = new JButton("Create");
        create.addActionListener(new ButtonListener());
        end.add(create);
        display.add(end, BorderLayout.SOUTH);
        
        content.add(display);
        
        //Display the window.
        frame.setLocationRelativeTo(MainWindow.frame);
        frame.setPreferredSize(new Dimension(600, 600));
        frame.pack();
        frame.setVisible(true);        
	}
	
	/**
	 * Checks to see if the current selected table is valid to add to the data set.
	 * 
	 * @return		boolean indicating if table name is valid
	 */
	public static boolean validTable() {
		boolean exists = false;
		
		for(int i = 0; i < MainWindow.tnames.length; i++) {
			if(MainWindow.tnames[i] != null) {
				//Check for duplicate
				if(selectedValue.equals(MainWindow.tnames[i])) {
					exists = true;
				}
			}
		}
		
		return exists;
	}

  	/**
	 * Listener to allow user to finalize and create the dataset.
	 */
    static class ButtonListener implements ActionListener {
       	/**
    	 * Used to catch when the create button is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		
    		int size = listModel.size();
    		boolean dup = false;
    		
    		if(size != 0) {
    		
    			//If a name is provided
	    		if(!name.getText().equals("")) {
	    			
	    			MainWindow.begin2.setEnabled(true);
	    			
	    		    Object[] tables = listModel.toArray();
	    			String dname = name.getText();
	    			
	    			Object[] dupcheck = MainWindow.setModel.toArray();
	    			
	    			for(int i = 0; i < dupcheck.length; i++) {
	    				if(dname.equals(dupcheck[i])) {
	    					dup = true;
	    				}
	    			}
	    			
	    			//If no duplicat exists, create set and add to set list
	    			if(!dup) {
		    			DataSet ex = new DataSet(dname, tables, setsIndex);
		    			sets[setsIndex] = ex;
		    			setsIndex++;
			        	
		    			if(MainWindow.setModel.firstElement().equals("(Empty)"))
		    				MainWindow.setModel.removeElement("(Empty)");
		    			
			        	MainWindow.setModel.addElement(dname);
			        	MainWindow.datasets.revalidate();
			        	MainWindow.datasets.repaint();
			        	
			        	MainWindow.desTable.setEnabled(true);
			        	
			        	frame.setVisible(false);
	    			}
	    			else {
		    			JOptionPane.showMessageDialog(frame, "Dataset already exists named: "+dname);
	    			}
	    		}
	    		else {
	    			JOptionPane.showMessageDialog(frame, "You must name the dataset");
	    		}
    		}
    		else {
    			JOptionPane.showMessageDialog(frame, "You must add tables to the dataset");
    		}
    	}
    }
    
  	/**
	 * Listener to allow user to add items to the dataset when "add" is pressed.
	 */
    static class ButtonListener2 implements ActionListener {
       	/**
    	 * Used to catch when the add button is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		Object[] temp; 
    		boolean dup = false;
			boolean exists = false;
    		
    		if(listModel.size() > 0) {
    			temp = listModel.toArray();
    			
    			for(int i = 0; i < listModel.size(); i++) {
    				if(selectedValue.equals(temp[i])) {
    					dup = true;
    				}
    			}
    				
    			//If no duplicate exists, add to list
    			if(!dup) {
    				    				
    				exists = validTable();
    				
    				if(exists)
    					listModel.addElement(selectedValue);
    				else
    		    		JOptionPane.showMessageDialog(frame, "You can only add tables to the dataset");
    			}
    			else {
		    		JOptionPane.showMessageDialog(frame, "A duplicate exists in the list");
    			}
    		}
    		else {
				exists = validTable();
				
				if(exists)
					listModel.addElement(selectedValue);  
				else
		    		JOptionPane.showMessageDialog(frame, "You can only add tables to the dataset");
			}
    	}
    }
    
  	/**
	 * Listener to allow user to remove items to the dataset when "rem" is pressed.
	 */
    static class ButtonListener3 implements ActionListener {
       	/**
    	 * Used to catch when the remove button is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		listModel.removeElement(selectedValue2);
    	}
    }
}
