package seniorproj;

import java.sql.*;
import java.util.*;

import oracle.jdbc.driver.OracleDriver;

/**
 * Contains the lower level methods dealing with the database and constructing of
 * sql queries.
 * 
 * @author Steven Weigand
 *
 */
public class DBConnect {
	
	/**
	 * The database connection.
	 */
	static Connection con = null;
	
	/**
	 * The possible columns in the dataset.
	 */
	static Column columns[] = null;
	
	/**
	 * The tables being referenced.
	 */
	static String[] refTables = null;
	
	/**
	 * The names being referenced.
	 */
	static String[] refNames = null;
	
	/**
	 * The index tracking the String arrays.
	 */
	static int tindex = 0;
	
	/**
	 * Stores sql.
	 */
	static String main = null;
	
	/**
	 * The columns being used by the user.
	 */
	static Column[] used = null;
	
	/**
	 * The number of columns.
	 */
	static int colnum = 0;
	
	/**
	 * The path of joins for the where clause.
	 */
	static String[] path = null;
	
	/**
	 * The path of joins in object form.
	 */
	static KeyPath[] paths = null;
	
	/**
	 * The index for paths.
	 */
	static int path_index = 0;

	/**
	 * Connects the user to the database.
	 * 
	 * @param username		Student's username
	 * @param password		Student's password
	 * @param db			Student's chosen db
	 * @return				boolean indicating if login was successful or not.
	 */
    public static boolean connection(String username, String password, String db) {
        
        try {
        	//Connect to database account
            DriverManager.registerDriver(new OracleDriver());

            con = DriverManager.getConnection (
                    "jdbc:oracle:thin:@hercules.csc.calpoly.edu:1522:ora10g", username, password);  
                        
            //Set auto commit to false
            con.setAutoCommit(false);      
            MainWindow.tnames = getTableNames();
            paths = new KeyPath[100];
        }
        catch(SQLException e) {
        	return false;
        }
        
        return true;
        
    }
    
    /**
     * Retrieves all of the tables available to the user from his/her account.
     * 
     * @return		Array containing available table names in the user's account.
     */
	public static String[] getTableNames() {
		
		int i = 1;
		String tables[] = new String[100];
		String query = "select table_name from user_tables";
		
		Statement stmt; 
        ResultSet rset;
        
        try {
        	stmt = con.createStatement();
            rset = stmt.executeQuery(query);
            
            //Save each table in user account to an array
            while (rset.next ()) { 
            	tables[i-1] = rset.getString("TABLE_NAME");
            	++i;
            }
        }
        catch(SQLException e) {
            System.out.println("ERROR: "+e.getMessage());
        }
        return tables;
	}
	
	/**
	 * Retrieves the columns for a given table name.
	 * 
	 * @param tablename	Name of table to retrieve columns for
	 * 
	 * @return			Array containg all columns in given table
	 */
	public static Column[] getColumns(String tablename) {
		
		String names[] = null;
		String types[] = null;
		int precision[] = null;
		int length[] = null;
		int scale[] = null;
        ResultSet rset;
		
		try {
	        // Create a result set
	        Statement stmt = con.createStatement();
	        ResultSet rs = stmt.executeQuery("SELECT * FROM "+tablename);
	    
	        // Get result set meta data
	        ResultSetMetaData rsmd = rs.getMetaData();
	        int numColumns = rsmd.getColumnCount();
	        
	        //Arrays for gathering column data
	        names = new String[numColumns];
	        types = new String[numColumns];
	        precision = new int[numColumns];
	        length = new int[numColumns];
	        scale = new int[numColumns];
	        columns = new Column[numColumns];
	    
	        for(int i = 1; i <= numColumns; i++) {
	        	names[i-1] = rsmd.getColumnName(i);
	        }
	        
	        //Execute query
            rset = stmt.executeQuery("select data_type, data_length, data_precision, data_scale " +
            		"                 from user_tab_columns where table_name = '"+tablename.toUpperCase()+"'");
            
            int i = 1;
            //While rows still exist
            while (rset.next ()) { 
            	types[i-1] = rset.getString("Data_Type");
            	precision[i-1] = rset.getInt("Data_Precision");
            	length[i-1] = rset.getInt("Data_Length");
            	scale[i-1] = rset.getInt("Data_Scale");

            	columns[i-1] = new Column(names[i-1], tablename, types[i-1], length[i-1], precision[i-1], scale[i-1]);
            	++i;
            }
	        
	    } 
		catch (SQLException e) {
	    	System.out.println("Error: "+e.getMessage());
	    }
		
	    return columns;
	}
		
	/**
	 * Creates the create table code for the main warehouse table.
	 * 
	 * @param name		Name of the central warehouse table
	 * @return			String containing the sql statement
	 */
	public static String createWarehouse(String name) {
		
		String create = "create table "+name+" (\n";
		String key = " primary key (";
		boolean last = false;
		boolean first = true;
		ResultSet rset = null;
		DataSet data = null;
		Object[] names = null;
		refTables = new String[100];
		refNames = new String[100];
		tindex = 0;
		String[] pkeys = new String[100];
		int kindex = 0;
		used = new Column[100];
		colnum = 0;
		boolean isKey = false;
		String code = "";
		
		//Get db metadata
		DatabaseMetaData dbmeta = null;
		try {
			dbmeta = con.getMetaData();
		} catch (SQLException e) {
            System.out.println("Error: "+e.getMessage());
		}
		
		//Determine what data set is being worked with
		for(int i = 0; i < CreateSet.setsIndex;  i++) {
			if(CreateSet.sets[i].getName() == MainWindow.selectedValue)
				data = CreateSet.sets[i]; 
		}
		
		//Find primary keys of tables
		names = data.getNames();
		for(int i = 0; i < names.length; i++) {
			if(names[i] != null) {
				try {
					rset = dbmeta.getPrimaryKeys(null, null, (String)names[i]);
					while(rset.next()) {
						pkeys[kindex++] = rset.getString("COLUMN_NAME");
					}
				} 
				catch (SQLException e) {
		            System.out.println("Error: "+e.getMessage());
				}
			}
		}
		
		//Figure out what tables are used and need to be added to warehouse
		for(int i = 0; i < SelectAttr.colindex; i++) {
			for(int j = 0; j < SelectAttr.key.length; j++) {
				
				StringTokenizer tok = new StringTokenizer((String) SelectAttr.key[j], ".");
				String tab = tok.nextToken();
				String nam = tok.nextToken();
				
				if(nam.equals(SelectAttr.allColumns[i].getName())) {
					if(tab.equals(SelectAttr.allColumns[i].getTname())) {
						used[colnum++] = SelectAttr.allColumns[i];
						
						if(first) {
							key += SelectAttr.allColumns[i].getName();
							first = false;
						}
						else {
							key += (", ");
							key += SelectAttr.allColumns[i].getName();
						}
					}
				}
			}
		}
		key += ") );\n";
		
		//Construct warehouse table and figure out what tables are used
		for(int i = 0; i < colnum; i++) {
			if(i == colnum-1) {
				last = true;
			}
			
			//Check to see if column is a primary key
			for(int j = 0; j < kindex; j++) {

				if(pkeys[j].equals(used[i].getName())) {
					refNames[tindex] = used[i].getTname();
					refTables[tindex++] = tableCode(used[i].getTname());
					isKey = true;
				
				}
			}
			
			//Construct create table statment for main table based on type
			create += " "+used[i].getName()+" "+used[i].getType()+"(";
			if(used[i].getPrec() == 0) {
				if(last) {
					create += used[i].getLength()+") ";
					if(isKey)
						create += "references "+used[i].getTname()+"("+used[i].getName()+"), \n";
					else
						create += ", \n";
				}
				else {
					create += used[i].getLength()+") ";
					if(isKey)
						create += "references "+used[i].getTname()+"("+used[i].getName()+"), \n";
					else
						create += ", \n";
				}
			}
			else {
				create += used[i].getPrec(); 
				
				if(used[i].getScale() != 0) {
					create += ","+used[i].getScale();
				}
				
				if(last)	{
					create += ")";
				    if(isKey) 
				    	create += " references "+used[i].getTname()+"("+used[i].getName()+"), \n";
				    else 
				    	create += ", \n";
				}
				else
					create += ")";
					if(isKey)		
						create += " references "+used[i].getTname()+"("+used[i].getName()+"), \n";
					else
						create += ", \n";
			}

			isKey = false;
		}
		
		Column temp;
		
		//Dependent Variables
		for(int i = 0; i < SelectDependents.vindex; i++) {
			
			temp = SelectDependents.vars[i].getColumn();
			create += " "+SelectDependents.vars[i].getAlias()+" "+temp.getType();
			
			if(temp.getPrec() == 0) {
				create += "("+temp.getLength()+"), \n";
			}
			else {
				create += "("+temp.getPrec(); 
				
				if(temp.getScale() != 0) {
					create += ","+temp.getScale();
				}
			    create += "), \n";
			}
		}
		
		create += key;
		main = create;
		
		for(int i = 0; i < tindex; i++) {
			code += refTables[i];
			code += "\n\n";
		}
		
		code += create;
				
		return code;
	}
	
	/**
	 * Creates table code for a given table. This is not called by the center of the star schema.
	 * 
	 * @param name		Name of table to create the statement for
	 * @return			String containing sql statement
	 */
	public static String tableCode(String name) {
		
		Column[] used = getColumns(name);
		String[] pkeys = new String[100];
		int kindex = 0;
		ResultSet rset = null;
		boolean isKey = false;
		boolean last = false;
		boolean first = true;
		String create = "create table "+name+" (\n";
		String key = " primary key (";

		//Get db meta data
		DatabaseMetaData dbmeta = null;
		try {
			dbmeta = con.getMetaData();
		} catch (SQLException e) {
            System.out.println("Error: "+e.getMessage());
		}
		
		//Get primary keys
		try {
			rset = dbmeta.getPrimaryKeys(null, null, (String)name);
			while(rset.next()) {
				pkeys[kindex++] = rset.getString("COLUMN_NAME");
			}
		} 
		catch (SQLException e) {
            System.out.println("Error: "+e.getMessage());
		}
		
		//Construct create table statements for tables with primary keys represented in dw
		for(int i = 0; i < used.length; i++) {
			if(i == used.length-1) {
				last = true;
			}
			
			for(int j = 0; j < kindex; j++) {
				if(pkeys[j].equals(columns[i].getName())) {
					if(first) {
						first = false;
					}
					else {
						key += ", ";
					}
					
					//isKey = true;
				    if(last)
				    	key += columns[i].getName();
				    else
				    	key += columns[i].getName();					
				}
			}
			
			//Create row by row by type
			create += " "+used[i].getName()+" "+used[i].getType()+"(";
			if(used[i].getPrec() == 0) {
				if(last) {
					create += used[i].getLength()+") ";
					if(isKey)
						create += "references "+used[i].getTname()+"("+used[i].getName()+"), \n";
					else
						create += ", \n";
				}
				else {
					create += used[i].getLength()+") ";
					if(isKey)
						create += "references "+used[i].getTname()+"("+used[i].getName()+"), \n";
					else
						create += ", \n";
				}
			}
			else {
				create += used[i].getPrec(); 
				
				if(used[i].getScale() != 0) {
					create += ","+used[i].getScale();
				}
				
				if(last)	{
					create += ")";
				    if(isKey) 
				    	create += " references "+used[i].getTname()+"("+used[i].getName()+"), \n";
				    else 
				    	create += ", \n";
				}
				else
					create += ")";
					if(isKey)		
						create += " references "+used[i].getTname()+"("+used[i].getName()+"), \n";
					else
						create += ", \n";
			}

			isKey = false;
		}
		
		key += ") );";
		create += key;
				
		return create;
	}
	
	/**
	 * Creates the insert statements to fill a table.
	 * 
	 * @param tablename		Name of the table
	 * @param columns		Columns of the given table
	 * @return				The sql for the insert statements
	 */
	public static String fillTable(String tablename, Column[] columns) {
		boolean last = false;
		
		String select = "insert into "+tablename+"\n (";
		
		//Create insert line
		for(int i = 0; i < columns.length; i++) {
			if(i == columns.length-1)
				last = true;
			if(last)
				select += columns[i].getName()+") \n ";
			else
				select += columns[i].getName()+", ";
		}
		
		last = false;
		select += "select ";
		
		//Create a select line
		for(int i = 0; i < columns.length; i++) {
			if(i == columns.length-1)
				last = true;
			if(last)
				select += columns[i].getTname()+"."+columns[i].getName()+"\n ";
			else
				select += columns[i].getTname()+"."+columns[i].getName()+", ";
		}
			
		last = false;
		select += "from "+tablename+"\n";	
		
		return select;
	}
	
	/**
	 * Checks to see if the given name already exists in array.
	 * 
	 * @param names		names of currently stored columns
	 * @param index		number of stored columns to search
	 * @param name		name of the column
	 * @return			boolean indicating if a duplicate exists or not
	 */
	public static boolean checkDup(String[] names, int index, String name) {
		
		for(int i = 0; i < index; i++) {
			if(names[i] != null) {
				if(names[i].equals(name)) {
					return true;
				}
			}
		}
		
		return false;
	}
	
	/**
	 * Figure out the path between two tables in a database.
	 * 
	 * @param keys		Foreign keys of all tables.
	 * @param keynum	Number of keys.
	 * @param from		Start table.
	 * @param to		End table.
	 * @param path		Path so far
	 * @param pindex	Number of steps in path so far.
	 */
	public static void getPath(TableKeys[] keys, int keynum, String from, String to, String[] path, int pindex) {
		
		System.out.println("TO: "+to+" FROM: "+from);
		
		int index = -1;
		int index2 = -1;
		
		boolean connected = false;
		
		/* Find key for to table */
		for(int i = 0; i < keynum; i++) {
			if(keys[i].getName().toUpperCase().equals(to.toUpperCase())) {
				index = i;
			}
		}
		
		/* Find key for from table */
		for(int i = 0; i < keynum; i++) {
			if(keys[i].getName().toUpperCase().equals(from.toUpperCase())) {
				index2= i;
			}
		}
		
		/* Check to see if tables are connected */
		for(int i = 0; i < keys[index].tabkeys.length; i++) {
			if(keys[index].tabkeys[i] != null) {
				if(!keys[index].visited) {
					/*if(pindex > 0) {
						System.out.println("1 COMPARISONS: "+keys[index].tabkeys[i]+" WITH "+path[0]);

						if(keys[index].tabkeys[i].toUpperCase().equals(path[0].toUpperCase())) {
							connected = true;
						}
					}
					else {*/

						if(keys[index].tabkeys[i].toUpperCase().equals(from.toUpperCase())) {
							connected = true;
						}
					//}
				}
			}
		}
		
		/* Check other way key just in case */
		if(!connected) {
			for(int i = 0; i < keys[index2].tabkeys.length; i++) {
				if(keys[index2].tabkeys[i] != null) {
					if(!keys[index2].visited) {
						/*if(pindex > 0) {

							if(keys[index].tabkeys[i].toUpperCase().equals(path[0].toUpperCase())) {
								connected = true;
							}
						}
						else {*/

							if(keys[index2].tabkeys[i].toUpperCase().equals(to.toUpperCase())) {
								connected = true;
							}
						//}
					}
				}
			}			
		}

		/* If connected, create and save path */
		if(connected) {
			path[pindex++] = to;
			path[pindex++] = from;

			KeyPath kpath = new KeyPath(from, to, path, pindex, keys, keynum);
			paths[path_index++] = kpath;
		}
		/* Else find path of all keys */
		else {
			String previous[] = new String[100];
			
			for(int i = 0; i < pindex; i++) {
				previous[i] = path[i];
			}
			
			previous[pindex++] = to;
			
			for(int i = 0; i < keys[index].tabkeys.length; i++) {
				if(keys[index].tabkeys[i] != null) {
					if(!keys[index].tabkeys[i].toUpperCase().equals(to.toUpperCase())) {
						getPath(keys, keynum, from, keys[index].tabkeys[i], previous, pindex);
					}	
				}
			}
		}
	}
	
	/**
	 * Creates the where clause for the sql by examining the foreign keys of attributes neighbors.
	 * 
	 * @param var		Dependent variable to base traversal on
	 * @return			Return String containing where clause
	 */
	public static String traverseFKey() {
		
		TableKeys keys[] = new TableKeys[100];
		int kindex = 0;
		
		String path[] = new String[100];
		
		ResultSet exp = null;
		ResultSet imp = null;
		boolean in = false;
		
		//Get db metadata
		DatabaseMetaData dbmeta = null;
		
		try {
			dbmeta = con.getMetaData();
		} catch (SQLException e) {
            System.out.println("Error: "+e.getMessage());
		}
		
		//Get list of tables that need to be connected
		String tabs[] = new String[100];
		String allTabs[] = new String[100];
		int tabindex = 0;
		
		//Get all tables available
		allTabs = getTableNames();
		
		//Independent variables
		for(int i = 0; i < SelectAttr.varindex; i++) {
			if(!checkDup(tabs, tabindex, (String)SelectAttr.tcolumns[i])) {
				tabs[tabindex++] = (String)SelectAttr.tcolumns[i];
			}
		}
		
		//Dependent Variables
		for(int i = 0; i < SelectDependents.vindex; i++) {
			if(!checkDup(tabs, tabindex, SelectDependents.vars[i].getColumn().getTname())) {
				tabs[tabindex++] = SelectDependents.vars[i].getColumn().getTname();
			}		
		}
		
		//Get the foreign keys for each table
		for(int i = 0; i < allTabs.length; i++) {
			if(allTabs[i] != null) {
				
				String tab = allTabs[i];
				
				String tnames[] = new String[100];
				String pnames[] = new String[100];
				String colnames[] = new String[100];
				int tnindex = 0;
				
				/* Retrieve foreign key info from database */
				try {
					exp = dbmeta.getExportedKeys(con.getCatalog(), null, tab);
					
					while(exp.next()) {
						if(exp != null) {
							in = true;
							
							tnames[tnindex] = exp.getString("FKTABLE_NAME");
							pnames[tnindex] = exp.getString("PKTABLE_NAME");
							colnames[tnindex++] = exp.getString("FKCOLUMN_NAME");
							
						}
					}
					exp.close();
					
					if(!in) {
						imp = dbmeta.getImportedKeys(con.getCatalog(), null, tab);
						
						while(imp.next()) {
							if(imp != null) {
													
								tnames[tnindex] = imp.getString("FKTABLE_NAME");
								pnames[tnindex] = imp.getString("PKTABLE_NAME");
								colnames[tnindex++] = imp.getString("FKCOLUMN_NAME");
									
							}
						}
						imp.close();
					}
				}
				catch(SQLException e) {
		            System.out.println("ERROR: "+e.getMessage());
				}
				
				if(tnindex > 0) {
					keys[kindex++] = new TableKeys(tab, tnames, colnames, pnames);
				}
				
				in = false;
			}
		}

		/* Get path for each set of tables */
		for(int i = 1; i < tabindex; i++) {
			getPath(keys, kindex, tabs[i-1], tabs[i], path, 0);
		}
			

		return null;
	}
	
	/**
	 * Create an instance of the where clause for two tables.
	 * 
	 * @param to		Start table
	 * @param from		End table
	 * @param keys		Keys for all tables
	 * @return			String containing statement such as "STUDENT.SID = ENROLLMENT.SID"
	 */
    public static String pieceClause(String to, String from, TableKeys keys[]) {
    	
    	String clause = null;
    	TableKeys to_ = null;
    	TableKeys from_ = null;
    	
    	/* Get key for to table */
		for(int i = 0; i < keys.length; i++) {
			if(keys[i] != null) {
				if(keys[i].getName().toUpperCase().equals(to.toUpperCase())) {
					to_ = keys[i];
					break;
				}
			}
		}
		
		/* Get key for from table */
		for(int i = 0; i < keys.length; i++) {
			if(keys[i] != null) {
				if(keys[i].getName().toUpperCase().equals(from.toUpperCase())) {
					from_ = keys[i];
					break;
				}			
			}
		}
		
		/* Find matching keys */
		for(int i = 0; i < to_.tabkeys.length; i++) {
			if(to_.tabkeys[i] != null) {
				for(int j = 0; j < from_.tabkeys.length; j++) {
					if(from_.tabkeys[j] != null) {
						if(to_.tabkeys[i].toUpperCase().equals(from_.tabkeys[j].toUpperCase())) {
							Column to_col[] = getColumns(to);
							Column from_col[] = getColumns(from);
							Column tcomp = null;
							Column fcomp = null;
							
							/* Retrieve matching columns */
							for(int k = 0; k < to_col.length; k++) {
								if(to_col[k] != null) {
									if(to_col[k].getName().toUpperCase().equals(to_.colkeys[i].toUpperCase())) {
										tcomp = to_col[k];
									}
								}
							}
							
							for(int k = 0; k < from_col.length; k++) {
								if(from_col[k] != null) {
									if(from_col[k].getName().toUpperCase().equals(from_.colkeys[j].toUpperCase())) {
										fcomp = from_col[k];
									}
								}
							}
							
							/* Check to see if columns match */
							if(to_.tabpkeys[i].toUpperCase().trim().equals(from_.tabpkeys[j].toUpperCase().trim())) {
								//if(to.toUpperCase().equals(from_.tabkeys[j].toUpperCase())) {
									if(!to_.tabkeys[i].equals(to)) {
										clause = to+"."+to_.colkeys[i]+" = "+from_.tabkeys[j]+"."+from_.colkeys[j];
										break;
									}
									else {
										clause = to_.tabkeys[i]+"."+to_.colkeys[i]+" = "+from+"."+from_.colkeys[j];
										break;
									}
								/*}
								else if(to_.tabkeys[i].toUpperCase().equals(from.toUpperCase())) {
									if(!to_.tabkeys[i].equals(to)) {
										clause = to+"."+to_.colkeys[i]+" = "+from_.tabkeys[j]+"."+from_.colkeys[j];
									}
									else {
										clause = to_.tabkeys[i]+"."+to_.colkeys[i]+" = "+from+"."+from_.colkeys[j];
									}
								}*/
							}
						}
					}
				}
			}
		}
		
		return clause;
    	
    }
	/**
	 * Creates the insert statement for the main table of the data warehouse.
	 * @return		String containing the sql code
	 */
	public static String starTable() {
		
		String where = "";
		boolean last = false;
		boolean first = true;
		String[] names = new String[100];
		int nindex = 0;
		
		String select = "\ninsert into "+SelectAttr.tablename+"\n (";
		
		//Indpendent variables
		for(int i = 0; i < SelectAttr.varindex; i++) {
			select += SelectAttr.columns[i]+", ";
		}
		
		//Dependent variables
		for(int i = 0; i < SelectDependents.vindex; i++) {
			if(SelectDependents.vindex-1 == i)
				last = true;
			if(last)
				select += SelectDependents.vars[i].getAlias()+")\n ";
			else
				select += SelectDependents.vars[i].getAlias()+" ,";
		}
		
		last = false;
		select += "select ";
		
		//Indpendent variables
		for(int i = 0; i < SelectAttr.varindex; i++) {
			select += SelectAttr.tcolumns[i]+"."+SelectAttr.columns[i]+", ";
		}
		
		//Dependent variables
		for(int i = 0; i < SelectDependents.vindex; i++) {
			if(SelectDependents.vindex-1 == i)
				last = true;
			if(last) {
				if(SelectDependents.vars[i].getFlag() == 0)
					select += SelectDependents.vars[i].getAgg()+"("+SelectDependents.vars[i].getColumn().getTname()+"."+SelectDependents.vars[i].getColumn().getName()+")\n";
				else
					select += SelectDependents.vars[i].getAgg()+"\n";
			}
			else {
				if(SelectDependents.vars[i].getFlag() == 0)
					select += SelectDependents.vars[i].getAgg()+"("+SelectDependents.vars[i].getColumn().getTname()+"."+SelectDependents.vars[i].getColumn().getName()+" ,";
				else
					select += SelectDependents.vars[i].getAgg()+" ,";
			}
		}
		last = false;		
		
		//From line
		select += " from ";
		
		
		for(int i = 0; i < CreateSet.sets[CreateSet.setsIndex-1].getNames().length; i++) {
			if(CreateSet.sets[CreateSet.setsIndex-1].getNames()[i] != null) {
				if(first)
					select += CreateSet.sets[CreateSet.setsIndex-1].getNames()[i];
				else
					select += ", "+CreateSet.sets[CreateSet.setsIndex-1].getNames()[i];
			}

			first = false;
		}
		
		//WHERE CLAUSE
		select += "\n where ";
		for(int i = 0; i < SelectDependents.vindex; i++) {
			traverseFKey();
		}
		
		String clauses[] = new String[100];
		int clindex = 0;
		first = true;
		
		//Piece path into clause
		for(int i = 0; i < path_index; i++) {
			TableKeys keys[] = paths[i].getKeys();
			
			for(int j = 1; j < paths[i].slen; j++) {
				String temp = pieceClause(paths[i].steps[j-1], paths[i].steps[j], keys);
				if(!checkDup(clauses, clindex, temp)) {
					if(first)
						select += temp;
					else
						select += " and "+temp;
				}
				first = false;
			}
		}
		
		//Group by all independent variables
		first = true;
		select += "\n group by ";
		for(int i = 0; i < SelectAttr.varindex; i++) {
			if(first)
				select += SelectAttr.tcolumns[i]+"."+SelectAttr.columns[i];
			else
				select += ", "+SelectAttr.tcolumns[i]+"."+SelectAttr.columns[i];
			
			first = false;
		}
		
		select += ";\n";
			
		return select;
	}
	
	/**
	 * Runs given sql code on the database and provides positive and negative feedback.
	 * @param code		sql to execute
	 * @return			String containing database result of sql execution
	 */
	public static String execute(String code) {
		String result = null;
		try {
	        Statement stmt = con.createStatement();
	        stmt.executeUpdate(code);	
	        result = "Table created and filled succesfully!";
		}
		catch (SQLException e) {
	    	result = "Error: "+e.getMessage();
	    }
		
		return result;
	}
}

