package seniorproj;

/**
 * Creates a dependent variable object containing information regarding the column alias, 
 * aggregate, column itself.
 * 
 * @author Steven Weigand
 */
public class DependentVar {
	
	String alias;
	String aggregate;
	Column col;
	int flag;
	
	/**
	 * DependentVar constructor for initializing and creating a DependentVar object.
	 * 
	 * @param alias		String containing name of var
	 * @param aggregate Name of aggregate function
	 * @param col		Column containing var information
	 * @param flag		Flag to indicate if custom or not
	 */
	public DependentVar(String alias, String aggregate, Column col, int flag) {
		this.alias = alias;
		this.aggregate = aggregate;
		this.col = col;
		this.flag = flag;
	}
	
	/**
	 * Retrieve the dependent variable's column.
	 * 
	 * @return		Column stored in var
	 */
	public Column getColumn() {
		return col;
	}
	
	/**
	 * Retrieve the dependent variable's alias.
	 * 
	 * @return		String alias of the var
	 */
	public String getAlias() {
		return alias;
	}
	
	/**
	 * Retrieve the dependent variable's aggregate function.
	 * 
	 * @return		Function name stored in var
	 */
	public String getAgg() {
		return aggregate;
	}
	
	/**
	 * Retrieve the dependent variable's flag.
	 * 
	 * @return		Flag stored in var
	 */
	public int getFlag() {
		return flag;
	}
}
