package seniorproj;

public class KeyPath {

	String fromTable;
	String toTable;
	String steps[];
	int slen;
	TableKeys keys[];
	int klen;
	
	/**
	 * Default Column constructor for initialization.
	 */
	public KeyPath() {}
	
	/**
	 * Creates and initializes KeyPath object.
	 * 
	 * @param fromTable		Starting table of path
	 * @param toTable		Destination table of path
	 * @param steps			Tables in the path
	 * @param slen			Number of steps
	 * @param keys			Keys for tables
	 * @param klen			Number of keys
	 */
	public KeyPath(String fromTable, String toTable, String steps[], int slen, TableKeys[] keys, int klen) {
		this.fromTable = fromTable;
		this.toTable = toTable;
		this.steps = steps;
		this.slen = slen;
		this.keys = keys;
		this.klen = klen;
	}
	
	/**
	 * Determine if a path already contains a step.
	 * 
	 * @param step		Table to check
	 * @return
	 */
	public boolean hasStep(String step) {
		for(int i = 0; i < steps.length; i++) {
			if(steps[i] != null) {
				if(steps[i].equals(step)) {
					return true;
				}
			}
		}
		return false;
	}
	
	/**
	 * Retrieve the path's start table.
	 * 
	 * @return		String containing table name
	 */
	public String getFrom() {
		return fromTable;
	}
	
	/**
	 * Retrieve the path's destination table.
	 * 
	 * @return		String containing dest table.
	 */
	public String getTo() {
		return toTable;
	}
	
	/**
	 * Retrieve the Steps.
	 * 
	 * @return		String[] containing Steps.
	 */
	public String[] getSteps() {
		return steps;
	}
	
	/**
	 * Retrieve the Keys.
	 * 
	 * @return		String[] containing keys.
	 */
	public TableKeys[] getKeys() {
		return keys;
	}
}
