package seniorproj;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Creates a GUI for a user to observe the available tables
 * and data sets available to him/her. It also allows the user
 * to create a new data set as well as begin the creation of a 
 * data warehouse from a selected data set.
 * 
 * @author Steven Weigand
 */
public class MainWindow {
	
	/**
	 * Frame containing the GUI components.
	 */
	static JFrame frame = null;

	/**
	 * Boolean indicating whether or not the user has logged in.
	 */
	static boolean loggedin = false;
	
	/**
	 * Menubar containing the following menu items.
	 */
	static JMenuBar menu = null;
	
	/**
	 * 'File' menu to be placed on the menubar.
	 */
	static JMenu file = null;
	
	/**
	 * The login option under the 'File' menu.
	 */
	static JMenuItem login = null;
	
	/**
	 * The open option under the 'File' menu.
	 */
	static JMenuItem open = null;
	
	/**
	 * The close option under the 'File' menu.
	 */
	static JMenuItem close = null;
	
	/**
	 * List containing user tables.
	 */
	static JList ownTables = null;
	
	/**
	 * List containing user made data sets.
	 */
	static JList ownSets = null;
	
	/**
	 * Listmodel for adding new datasets to the ownSets list.
	 */
	static DefaultListModel setModel = null;
	
	/**
	 * The currently selected object on the ownTables list.
	 */
	static Object selectedValue = null;
	
	/**
	 * The currently selected object on the ownSets list.
	 */
	static Object selectedValue2 = null;
	
	/**
	 * Button allowing user to view tables within a data set.
	 */
	static JButton desTable = null;
	
	/**
	 * Button to begin creating a data set.
	 */
	static JButton begin = null;

	/**
	 * Button to begin creating a data warehouse.
	 */
	static JButton begin2 = null;
	
	/**
	 * Array for temporary storage of table columns.
	 */
    static Column temp[] = null;
    
	/**
	 * Array for storage of table names.
	 */
    static String[] tnames = null;
    
	/**
	 * Panel containing dataset GUI components.
	 */
    static JPanel datasets = null;
    
    /**
     * Creates the main window displaying the list of created data
     * sets and the list of tables on the user's account.
     */
	public static void createMainWindow() {
		
        frame = new JFrame("CSC 467: Main Window");

        //Create container
        Container content = frame.getContentPane();
        content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
        content.setBackground(Color.white);

        //Create menu bar and listeners
        menu = new JMenuBar();
        file = new JMenu("File");
        login = new JMenuItem("Login");
        open = new JMenuItem("Open");
        close = new JMenuItem("Close");
        
        login.addActionListener(new MenuListener());
        open.addActionListener(new MenuListener2());
        close.addActionListener(new MenuListener3());
        
        open.setEnabled(false);
        
        if(loggedin)
        	login.setEnabled(false);
        
        file.add(login);
        file.add(open);
        file.add(close);
        menu.add(file);
        
        //Panel for dataset list
        JPanel datasetsLabel = new JPanel();
        JLabel sets = new JLabel("Datasets");
        datasetsLabel.add(sets);
        datasetsLabel.setMaximumSize(new Dimension(1000, 30));
        content.add(datasetsLabel);
        
        datasets = new JPanel();
        content.add(datasets);
        
        JPanel setlist = new JPanel();
        
        //Create list to store data sets 
        setModel = new DefaultListModel();
        ownSets = new JList(setModel);
        ownSets.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        ownSets.addListSelectionListener(new ListSelectionListener() {
        	public void valueChanged(ListSelectionEvent evt) {
        		JList list = (JList)evt.getSource();
                selectedValue = list.getSelectedValue();
        	}
        });
        setModel.addElement("(Empty)");
        ownSets.setPreferredSize(new Dimension(150, 250));
        JScrollPane pane = new JScrollPane(ownSets);
		pane.setPreferredSize(new Dimension(150, 250));

        setlist.add(pane);
        content.add(setlist);
        
        //Create describe button for data sets
        JPanel desc = new JPanel();
        desTable = new JButton("Describe");
        desTable.addActionListener(new ButtonListener2());
        desc.add(desTable);
        
        desTable.setEnabled(false);
        
        content.add(desc);
        
        //Create list of available tables in tree form
        JPanel tablelist = new JPanel();
        
        DefaultMutableTreeNode root = new DefaultMutableTreeNode("Tables");
        DefaultMutableTreeNode parent;
        DefaultMutableTreeNode child;
        
        if(loggedin) {
	        //tnames = DBConnect.getTableNames();
			
			for(int i = 0; i < tnames.length; i++) {
				if(tnames[i]  != null) {
					Column[] names = DBConnect.getColumns(tnames[i]);
					parent = new DefaultMutableTreeNode(tnames[i]);
					for(int j = 0; j < names.length; j++) {
						if(names[j] != null) {
							child = new DefaultMutableTreeNode(names[j].getName());
							parent.add(child);
						}
					}
					root.add(parent);
				}
			}
        }
		
		JTree tree = new JTree(root);
		tree.setScrollsOnExpand(true);
		tree.setPreferredSize(new Dimension(150, 250));
        JScrollPane pane2 = new JScrollPane(tree);
		pane2.setPreferredSize(new Dimension(150, 250));

		tablelist.add(pane2);
		
        content.add(tablelist);

        //Panel for create button
        JPanel create = new JPanel();
        begin = new JButton("Create Dataset");
        begin.addActionListener(new ButtonListener());
       
        if(!loggedin)
        	begin.setEnabled(false);
        create.add(begin);
        
        begin2 = new JButton("Create Warehouse");
        begin2.addActionListener(new ButtonListener4());
        begin2.setEnabled(false);
        
        create.add(begin2);
        content.add(create);
        
        //Display the window.
        if(loggedin)
        	frame.setLocationRelativeTo(ConnectionGUI.frame);
        
        frame.setJMenuBar(menu);
        frame.setPreferredSize(new Dimension(350, 700));
		frame.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        frame.pack();
        frame.setVisible(true);  
	}
	
	/**
	 * Creates a button listener for the "Create Dataset" button.
	 */
    static class ButtonListener implements ActionListener {
       	/**
    	 * Used to catch when the create set button is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		MainWindow.begin.setEnabled(false);
    		CreateSet.createSetGUI();
    		MainWindow.begin.setEnabled(true);
    	}
    }
    
	/**
	 * Creates a button listener for the "Describe" button.
	 */
    static class ButtonListener2 implements ActionListener {
       	/**
    	 * Used to catch when the describe button is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		if(selectedValue == null || selectedValue == "(Empty)") {
    			JOptionPane.showMessageDialog(frame,"There are no datasets to choose from");
    		}
    		else {
    			Object temp[] = null; 
    			
    			for(int i = 0; i < CreateSet.setsIndex; i++) {
    				if(selectedValue.equals(CreateSet.sets[i].id))
    					temp = CreateSet.sets[i].tablenames;
    			}
    			
    			String message = selectedValue+" contains the following tables:\n";
    			for(int i = 0; i < temp.length; i++) {
    				if(temp[i] != null)
    					message += "      "+temp[i]+"\n";
    			}
    			
    			JOptionPane.showMessageDialog(frame, message);
    		}
    	}
    }
    
	/**
	 * (*UNUSED*) Creates a button listener for the "Describe Table" button.
	 */
    static class ButtonListener3 implements ActionListener {
       	/**
    	 * Used to catch when the create describe table is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		if(selectedValue2 == null) {
    			JOptionPane.showMessageDialog(frame, "Please select a table");
    		}
    		else {
    			Column[] names = DBConnect.getColumns((String)selectedValue2);
    			String message = selectedValue2+" contains the following columns:\n";
    			
    			for(int i = 0; i < names.length; i++) {
    				if(names[i] != null)
    					message += "      "+names[i].getName()+"\n";
    			}
    			
    			JOptionPane.showMessageDialog(frame, message);
    					        
    		}
    	}
    }
    
	/**
	 * Creates a button listener for the "Create Warehouse" button.
	 */
    static class ButtonListener4 implements ActionListener {
       	/**
    	 * Used to catch when the create warehouse button is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		if(selectedValue == null) {
    			JOptionPane.showMessageDialog(frame, "Please select a dataset");
    		}
    		else {
    			if(selectedValue.equals("(Empty)")) {
        			JOptionPane.showMessageDialog(frame, "Please create a dataset first");
    			}
    			else {
        			SelectAttr.createSelectAttrs();
    			}
    		}
    	}
    }
    
	/**
	 * Creates a menu listener for the "Login" item.
	 */
    static class MenuListener implements ActionListener {
       	/**
    	 * Used to catch when the login item is selected.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		ConnectionGUI.createConnectGUI();
    	}
    }
    
	/**
	 * Creates a menu listener for the "Open" item.
	 */
    static class MenuListener2 implements ActionListener {
       	/**
    	 * Used to catch when the open item is selected.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		ChooseFile choose = new ChooseFile();
    	}
    }
    
	/**
	 * Creates a menu listener for the "Close" item.
	 */
    static class MenuListener3 implements ActionListener {
       	/**
    	 * Used to catch when the close item is selected.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		System.exit(0);
    	}
    }
}
