package seniorproj;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;

/**
 * Creates GUI for the user to choose the independent variables
 * which will eventually become the primary key in the main table
 * of the data warehouse.
 * 
 * @author Steven Weigand
 */
public class SelectAttr {
	
	/**
	 * The frame containing the GUI components.
	 */
	static JFrame frame = null;
	
	/**
	 * Button allowing user to save independent vars and move to select dependent vars.
	 */
	static JButton create = null;
	
	/**
	 * List containing available columns to choose from.
	 */
	static JList attrs = null;
	
	/**
	 * List containing chosen columns to be independent vars.
	 */
	static JList chosen = null;
	
	/**
	 * ListModel to allow adding/removing from 'chosen' list.
	 */
	static DefaultListModel listModel = null;
	
	/**
	 * The currently selected object in the 'attrs' list.
	 */
	static Object selectedValue = null;
	
	/**
	 * The currently selected object in the 'chosen' list.
	 */
	static Object selectedValue2 = null;
	
	/**
	 * Array of all tables contained in current data set.
	 */
	static Object tables[] = null;
	
	/**
	 * Array of chosen columns.
	 */
	static Object columns[] = null;
	
	/**
	 * Array of chosen columns' associated tables.
	 */
	static Object tcolumns[] = null;
	
	/**
	 * Tracks size of 'columns' and 'tcolumns'.
	 */
	static int varindex = 0;
	
	/**
	 * Textfield containing name of user provided tablename.
	 */
	static JTextField name = null;
	
	/**
	 * Array of temporary storage of a table's columns.
	 */
    static Column temp[] = null;
    
	/**
	 * Array of all available columns.
	 */
    static Column allColumns[] = null;
    
	/**
	 * Tracks number of availalbe columns.
	 */
    static int colindex = 0;
    
	/**
	 * Holds chosen objects upon completion of ind. var selection.
	 */
	static Object[] key = null;
	
	/**
	 * String containing name of user provided tablename.
	 */
	static String tablename = null;
	
    /**
     * Tree node used to find the parent of 'selectedValue'.
     */
    static DefaultMutableTreeNode pnode = null;

    /**
     * Creates GUI layout for user to select independent variables from available
     * data set columns. After selecting, the user would move to GUI allowing for the
     * selection of dependent variables.
     */
	public static void createSelectAttrs() {
		
		colindex = 0;
		varindex = 0;
		
        frame = new JFrame("CSC 467: Choose Your Primary Key(s)");

        colindex = 0;
        Container content = frame.getContentPane();
        content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
        content.setBackground(Color.GRAY);
        
        JPanel display = new JPanel();
        display.setLayout(new BorderLayout());
        
        columns = new Object[100];
        tcolumns = new Object[100];

        //Enter name 
        JPanel enterName = new JPanel();
        JLabel choose = new JLabel("Table Name:");
        enterName.add(choose); 
        name = new JTextField(15);
        enterName.add(name);
        display.add(enterName, BorderLayout.NORTH);
        
        DefaultMutableTreeNode root;
        DefaultMutableTreeNode parent;
        DefaultMutableTreeNode child;
        
        //Retrieve data set and corresponding tables
        tables = CreateSet.sets[CreateSet.setsIndex-1].getNames();
        allColumns = new Column[100];
        
        //Tree of available tables and columns in data set
        root = new DefaultMutableTreeNode("Tables");
        
        for(int i = 0; i < tables.length; i++) {
        	if(tables[i] != null) {
        		temp = DBConnect.getColumns((String)tables[i]);
        		parent = new DefaultMutableTreeNode(tables[i]); 
        		
        		for(int j = 0; j < temp.length; j++) {
        			if(temp[j] != null) {
            			allColumns[colindex++] = temp[j];
            			child = new DefaultMutableTreeNode(temp[j].getName());
            			parent.add(child);
        			}
        		}
        		root.add(parent);
        	}
        }
        
        final JTree tree = new JTree(root);
        tree.setPreferredSize(new Dimension(250, 500));
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.CONTIGUOUS_TREE_SELECTION);

		//Listen for when the selection changes.
		tree.addTreeSelectionListener(new TreeSelectionListener() {
		    public void valueChanged(TreeSelectionEvent e) {
		        DefaultMutableTreeNode node = (DefaultMutableTreeNode)tree.getLastSelectedPathComponent();

		        if (node == null) {
		        	
		        }
		        else {
		        	selectedValue = node.getUserObject();
		        	pnode = (DefaultMutableTreeNode)node.getParent();
		        }

		    }
		});
        JScrollPane pane = new JScrollPane(tree);
        pane.setPreferredSize(new Dimension(250, 500));

        display.add(pane, BorderLayout.WEST);
        
        //Dependent variables
        JPanel dvariables = new JPanel();
        JLabel dvar = new JLabel("Choose Primary Key from "+MainWindow.selectedValue+" Set");
        dvariables.add(dvar);
        content.add(dvariables);
        
        //Buttons to control lists
        JPanel buttons = new JPanel();
        JPanel buttons1 = new JPanel();
        buttons1.setPreferredSize(new Dimension(800, 50));
        JPanel buttons2 = new JPanel();
        buttons2.setPreferredSize(new Dimension(800, 50));
        JPanel padding = new JPanel();
        padding.setPreferredSize(new Dimension(800, 100));

        JButton add = new JButton("Add >>");
        add.addActionListener(new ButtonListener2());
        buttons1.add(add);
        JButton rem = new JButton("<< Remove");
        rem.addActionListener(new ButtonListener3());
        buttons2.add(rem);
        buttons.add(padding);
        buttons.add(buttons1);
        buttons.add(buttons2);
        display.add(buttons, BorderLayout.CENTER);
        
        //List of selected columns
        JPanel setAttrs = new JPanel();
        listModel = new DefaultListModel();
        chosen = new JList(listModel);
        chosen.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        chosen.addListSelectionListener(new ListSelectionListener() {
        	public void valueChanged(ListSelectionEvent evt) {
        		JList list = (JList)evt.getSource();
                selectedValue2 = list.getSelectedValue();
        	}
        });
        chosen.setPreferredSize(new Dimension(250, 500));
        JScrollPane pane2 = new JScrollPane(chosen);
        pane2.setPreferredSize(new Dimension(250, 500));
        
        setAttrs.add(pane2);
        display.add(setAttrs, BorderLayout.EAST);
        
        //Button to create the warehouse
        JPanel start = new JPanel();
        create = new JButton("Select Dependents");
        create.addActionListener(new ButtonListener());
        start.add(create);
        display.add(start, BorderLayout.SOUTH);
        
        content.add(display);
        //Display the window.
        frame.setLocationRelativeTo(MainWindow.frame);
        frame.setPreferredSize(new Dimension(700, 600));
        frame.pack();
        frame.setVisible(true);        
	}
	
	/**
	 * Creates a button listener for the "Select Dependents" button.
	 */
    static class ButtonListener implements ActionListener {
       	/**
    	 * Used to catch when the Select Dependents button is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		int size = listModel.size();
    		
    		if(size != 0) {

	    		if(!name.getText().equals("")) {
	    			
	    			//Retrieve key and move to dependent variables
	    			tablename = name.getText();
	    			key = listModel.toArray();

	    			SelectDependents.createSelectDependents();

	        		frame.setVisible(false);

	    		}
	    		else {
	    			JOptionPane.showMessageDialog(frame, "You must name the table");
	    		}
    		}
    		else {
    			JOptionPane.showMessageDialog(frame, "You must add columns to the table");
    		}
       	}
    }
    
	/**
	 * Creates a button listener for the "Add" button.
	 */
    static class ButtonListener2 implements ActionListener {
       	/**
    	 * Used to catch when the Add button is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */    	public void actionPerformed(ActionEvent e) {
    		
    		boolean dup = false;
    		String temp;
    		
    		if(selectedValue != null) {
    			
    			Object[] dupcheck = listModel.toArray();
    			if(pnode != null) {
    				temp = pnode.toString()+"."+selectedValue;
    			
	    			if(pnode.toString().equals("Tables")) {
	        			JOptionPane.showMessageDialog(frame, "You can only add columns to the primary key");
	    			}
	    			else {
	    				//Check for dup
	        			for(int i = 0; i < dupcheck.length; i++) {
	        				if(temp.equals(dupcheck[i])) {
	        					dup = true;
	        				}
	        			}
	            		if(!dup) {
	            			//Add item to independent variable array
	                		listModel.addElement(temp);
	                		columns[varindex] = selectedValue;
	                		tcolumns[varindex++] = pnode.toString();
	            		}
	        			else {
	            			JOptionPane.showMessageDialog(frame, "A duplicate in the list exists");
	        			}
	    			}
    			}
    			else {
        			JOptionPane.showMessageDialog(frame, "You can only add columns to the primary key");    				
    			}
    		}
    	}
    }
    
	/**
	 * Creates a button listener for the "Remove" button.
	 */
    static class ButtonListener3 implements ActionListener {
       	/**
    	 * Used to catch when the Remove button is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		if(selectedValue2 != null) {
    			int remove = -1;
    			
    			for(int i = 0; i < varindex; i++) {
    				if(selectedValue2.equals(tcolumns[i]+"."+columns[i])) {
    					remove = i;
    				}
    			}
    			
    			for(int i = remove+1; i < varindex; i++) {
    				columns[i-1] = columns[i];
    				tcolumns[i-1] = tcolumns[i];
    			}
    			
    			listModel.removeElement(selectedValue2);

    		}
    	}
    }
}
