package seniorproj;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;

/**
 * Creates a GUI for a user to select and build his/her dependent
 * variables for a data warehouse. Allows options to create an alias
 * and customize the way the aggregate will be calculated
 * with all of the proper button action listeners.
 * 
 * @author Steven Weigand
 *
 */
public class SelectDependents {
	
    /**
     * The frame containing the GUI components.
     */
	static JFrame frame = null;
	
    /**
     * Array containing dependent variables.
     */
	static DependentVar[] vars = null;
	
    /**
     * Array containing String of dependent variables.
     */
	static String[] varCopy = null;
	
    /**
     * Index tracking the number of elements in vars.
     */
	static int vindex = 0;
	
    /**
     * Array containing full description of each dependent variable.
     */
	static Object[] full = null;
	
    /**
     * Button used to create warehouse state.
     */
	static JButton create = null;
	
    /**
     * Button used to add dependent variables to the list.
     */
	static JButton add = null;
	
    /**
     * Button used to remove dependent variables fromt the list.
     */
	static JButton remove = null;
	
    /**
     * Button used to create a custom aggregate function.
     */
	static JButton aggCreate = null;
	
    /**
     * Button used to go back to the previous state.
     */
	static JButton back = null;
	
    /**
     * List containing available columns.
     */
	static JList attrs = null;
	
    /**
     * List containing the newly created dependent variables.
     */
	static JList chosen = null;
	
    /**
     * Dropdown menu containing aggregate functions.
     */
	static JComboBox dropdown = null;
	
    /**
     * Listmodel used to allow adding and removing from a list.
     */
	static DefaultListModel listModel = null;
	
    /**
     * Current selected value in the attrs list.
     */
	static Object selectedValue = null;
	
    /**
     * Current selected value in the chosen list.
     */
	static Object selectedValue2 = null;
	
    /**
     * Current selected value in the dropdown menu.
     */
	static Object selectedValue3 = null;
	
    /**
     * Array of tables in the available dataset.
     */
	static Object tables[] = null;

    /**
     * Textfield containing the alias of a dependent variable.
     */
	static JTextField name = null;
	
    /**
     * Array for temporary storage of table columns.
     */
    static Column temp[] = null;
    
    /**
     * Array of all available columns in the dataset.
     */
    static Column allColumns[] = null;
    
    /**
     * Index to track number of available columns.
     */
    static int colindex = 0;
    
    /**
     * Tree node used to find the parent of 'selectedValue'.
     */
    static DefaultMutableTreeNode pnode = null;

    /**
     * Creates the GUI layout for a user to select columns and create
     * dependent variables from the supported SQL aggregate functions.
     */
	public static void createSelectDependents() {
		
		vindex = 0;
		colindex = 0;
		
        frame = new JFrame("CSC 467: Dependent Variables");
        boolean found = false;

        Container content = frame.getContentPane();
        content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
        content.setBackground(Color.GRAY);
        
        //Dropdown menu choices
        Object[] agg = {"(CHOOSE)","SUM", "COUNT", "MAX", "MIN", "AVG"};
        vars = new DependentVar[100];
        varCopy = new String[100];

        JPanel dvariables = new JPanel();
        JLabel dvar = new JLabel("Create Depedent Variables For "+SelectAttr.tablename);

        dvariables.add(dvar);
        content.add(dvariables);
        
        //Place primary key in GUI to notify user
        JPanel keylist = new JPanel();
        String keys = "Primary Key: ";
        
        for(int i = 0; i < SelectAttr.key.length; i++) {
        	keys += SelectAttr.key[i] + " ";
        }
        
        JLabel pkey = new JLabel(keys);
        keylist.add(pkey);
        content.add(keylist);
        
        JPanel display = new JPanel();
        display.setLayout(new BorderLayout());
        
        //Enter name here
        JPanel enterName = new JPanel();
        JLabel choose = new JLabel("Alias:");
        enterName.add(choose); 
        name = new JTextField(15);
        enterName.add(name);
        display.add(enterName, BorderLayout.NORTH);
                
        //Tree list of all available columns
        DefaultMutableTreeNode root;
        DefaultMutableTreeNode parent;
        DefaultMutableTreeNode child;
        
        tables = CreateSet.sets[CreateSet.setsIndex-1].getNames();
        allColumns = new Column[100];
        String check = null;
        
        root = new DefaultMutableTreeNode("Tables");
        
        for(int i = 0; i < tables.length; i++) {
        	if(tables[i] != null) {
        		temp = DBConnect.getColumns((String)tables[i]);
        		parent = new DefaultMutableTreeNode(tables[i]); 
        		
        		for(int j = 0; j < temp.length; j++) {
        			if(temp[j] != null) {
        				check = temp[j].getTname()+"."+temp[j].getName();
        				for(int k = 0; k < SelectAttr.key.length; k++) {
            				if(check.equals(SelectAttr.key[k])) {
            					found = true;
            				}
        				}
        				
        				if(!found) {
        					allColumns[colindex++] = temp[j];
            			
            				child = new DefaultMutableTreeNode(temp[j].getName());
            				parent.add(child);
        				}
        				found = false;
        			}
        		}
        		root.add(parent);
        	}
        }
        
        final JTree tree = new JTree(root);
        tree.setPreferredSize(new Dimension(250, 500));
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.CONTIGUOUS_TREE_SELECTION);

		//Listen for when the selection changes.
		tree.addTreeSelectionListener(new TreeSelectionListener() {
		    public void valueChanged(TreeSelectionEvent e) {
		        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
		                           tree.getLastSelectedPathComponent();
		        
		        if (node == null) return;
		        else {
		        	selectedValue = node.getUserObject();
		        	pnode = (DefaultMutableTreeNode)node.getParent();
		        }
		    }
		});
		
        JScrollPane pane = new JScrollPane(tree);
        pane.setPreferredSize(new Dimension(250, 500));

        display.add(pane, BorderLayout.WEST);
        
        JPanel aggs = new JPanel();
        JPanel pdrop = new JPanel();
        JPanel button0 = new JPanel();
        JPanel button1 = new JPanel();
        JPanel button2 = new JPanel();
        
        
        JLabel padding = new JLabel();
        padding.setPreferredSize(new Dimension(800, 100));
        aggs.add(padding);
        
        //Drop down menu
        dropdown = new JComboBox(agg);
        dropdown.addActionListener(new DropDownListener());
        pdrop.setPreferredSize(new Dimension(800, 50));
        pdrop.add(dropdown);
        
        //View state of warehouse
        aggCreate = new JButton("Customize Variable");
        aggCreate.addActionListener(new ButtonListener4());
        button0.setPreferredSize(new Dimension(800, 50));
        button0.add(aggCreate);
        
        //Add button
        add = new JButton("Add >>");
        add.addActionListener(new ButtonListener2());
        button1.setPreferredSize(new Dimension(800, 50));
        button1.add(add);
        
        //Remove button
        remove = new JButton("<< Remove");
        remove.addActionListener(new ButtonListener3());
        button2.setPreferredSize(new Dimension(800, 50));
        button2.add(remove);
        
        aggs.add(pdrop);
        aggs.add(button0);
        aggs.add(button1);
        aggs.add(button2);
        
        display.add(aggs);
        
        //List of selected columns
        JPanel setAttrs = new JPanel();
        listModel = new DefaultListModel();
        chosen = new JList(listModel);
        chosen.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        chosen.addListSelectionListener(new ListSelectionListener() {
        	public void valueChanged(ListSelectionEvent evt) {
        		JList list = (JList)evt.getSource();
                selectedValue2 = list.getSelectedValue();
        	}
        });
        chosen.setPreferredSize(new Dimension(250, 500));
        JScrollPane pane2 = new JScrollPane(chosen);
        pane2.setPreferredSize(new Dimension(250, 500));

        setAttrs.add(pane2);
        display.add(setAttrs, BorderLayout.EAST);
        
        //Button to create the warehouse
        JPanel start = new JPanel();
        create = new JButton("View State");
        create.addActionListener(new ButtonListener());
        back = new JButton("Back");
        back.addActionListener(new ButtonListener5());
        start.add(back);
        start.add(create);
        display.add(start, BorderLayout.SOUTH);
        
        content.add(display);
        
        //Display the window.
        frame.setLocationRelativeTo(SelectAttr.frame);
        frame.setPreferredSize(new Dimension(700, 600));
        frame.pack();
        frame.setVisible(true);        
	}
	
	/**
	 * Creates a button listener for the "View State" button.
	 */
    static class ButtonListener implements ActionListener {

       	/**
    	 * Used to catch when the view state button is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		
    		String sql = null;
    		
    		if(name.getText() != "") {
    			
        		int size = listModel.size();
        		
        		if(size != 0) {
    	    			    	    
        			full = new Object[size];
        			full = listModel.toArray();
	    			sql = DBConnect.createWarehouse(SelectAttr.tablename);
    	    		
    	    		TableAnalysis.createAndShowGUI7(sql);
    	    		frame.setVisible(false);
    	    	}
    	    	else {
    	    		JOptionPane.showMessageDialog(frame, "You must create a dependent variable first");
    	   		}
    		}
    		else {
    			JOptionPane.showMessageDialog(frame, "You must name the variable");
    		}
       	}
    }
    
	/**
	 * Creates a button listener for the "Add" button.
	 */
    static class ButtonListener2 implements ActionListener {
       	/**
    	 * Used to catch when the add button is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		
    		String element;
    		String newAgg;
    		String temp;
    		Column col = null;
    		boolean dup = false;
    		
    		if(selectedValue != null) {
    			
    			if(pnode != null) {
	        		element = pnode.toString()+"."+selectedValue;
	        		
	    			if(pnode.toString().equals("Tables")) {
	        			JOptionPane.showMessageDialog(frame, "You can dependent variables from columns");
	    			}
	    			else {
	            		if(selectedValue3 != null) {
	    		    		if(!selectedValue3.equals("(Choose)")) {
	    		    			newAgg = selectedValue3+"("+element+")";
	    		    			
	    		    			if(!name.getText().equals("")) {
	    		    				
	    		    				for(int i = 0; i < SelectAttr.colindex; i++) {
	    		    					temp = SelectAttr.allColumns[i].getTname()+"."+SelectAttr.allColumns[i].getName();
	    		    					if(element.equals(temp)) {
	    		    						col = SelectAttr.allColumns[i];
	    		    					}
	    		    				}
	    		    				
	    		    				newAgg += " as "+name.getText();
	    		    				
	    		    				Object[] dupcheck = listModel.toArray();
	
	    		    				for(int i = 0; i < dupcheck.length; i++) {
	    		    					if(newAgg.equals(dupcheck[i]))
	    		    						dup = true;
	    		    				}
	    		    				
	    		    				if(!dup) {
	    				    			listModel.addElement(newAgg);
	    				    			
	    			    				DependentVar var = new DependentVar(name.getText(), (String)selectedValue3, col, 0);
	    			    				varCopy[vindex] = newAgg;
	    			    				vars[vindex++] = var;
	    		    				}
	    		    				else {
	    				    			JOptionPane.showMessageDialog(frame, "A duplicate in the list exists");
	    		    				}
	    		    			}
	    		    			else {
	    			    			JOptionPane.showMessageDialog(frame, "Please give the variable an alias");
	    		    			}
	    		    		}
	    		    		else {
	    		    			JOptionPane.showMessageDialog(frame, "Please choose from the dropdown menu");
	    		    		}
	            		}
	    	    		else {
	    	    			JOptionPane.showMessageDialog(frame, "Please choose from the dropdown menu");
	    	    		}
	        		}	
	    		}	
    			else {
        			JOptionPane.showMessageDialog(frame, "You can dependent variables from columns");
    			}
    		}
    		else {
    			JOptionPane.showMessageDialog(frame, "Please choose a column first");
    		}
    	}
    }
    
	/**
	 * Creates a button listener for the "Remove" button.
	 */
    static class ButtonListener3 implements ActionListener {
       	/**
    	 * Used to catch when the remove button is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		
    		if(selectedValue2 != null) {
    			int remove = -1;
    			
    			for(int i = 0; i < vindex; i++) {
    				if(selectedValue2.equals(varCopy[i])) {
    					remove = i;
    				}
    			}
    			
    			for(int i = remove+1; i < vindex; i++) {
    				vars[i-1] = vars[i];
    			}
    			
    			listModel.removeElement(selectedValue2);
    		}
    	}
    }
    
	/**
	 * Creates a button listener for the "Custom Var" button.
	 */
    static class ButtonListener4 implements ActionListener {
       	/**
    	 * Used to catch when the create custom button is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		CustomDependent.createCustomDVar();
    	}
    }
    
	/**
	 * Creates a button listener for the "Back" button.
	 */
    static class ButtonListener5 implements ActionListener {
       	/**
    	 * Used to catch when the back button is pressed.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		SelectAttr.createSelectAttrs();
    		frame.setVisible(false);
    	}
    }
    
	/**
	 * Creates a dropdown listener for the aggregate drop down menu.
	 */
    static class DropDownListener implements ActionListener {
       	/**
    	 * Used to catch when the aggregate dropdown menu is selected.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		JComboBox box = (JComboBox)e.getSource();
    		selectedValue3 = box.getSelectedItem();
    	}
    }
}
