package seniorproj;

import javax.swing.*;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Creates a three-paned GUI consisting of three parts:
 * (1) Overview containing text description of warehouse state.
 * (2) SQL generated for creating the warehouse.
 * (3) A diagram of the warehouse state.
 * 
 * @author Steven Weigand
 */
public class TableAnalysis {
	
	/**
	 * The frame containing the GUI components.
	 */
	static JFrame frame = null;
	
	/**
	 * Menubar containing options to 'Execute SQL', 'Save', 'Export', 'Back'.
	 */
	static JMenuBar menu = null;
	
	/**
	 * Sql code of the current warehouse state.
	 */
	static String code = null;
	
	/**
	 * Boolean indicating if the warehouse has been constructed yet.
	 */
	static boolean warehouse = false;
	
	/**
	 * Creates GUI layout with three pane view for user to browse warehouse
	 * state in three ways as well as execute the code.
	 * 
	 * @param sql		The sql code of the warehouse
	 */
	public static void createAndShowGUI7(String sql) {
		
		warehouse = true;
		
        frame = new JFrame("CSC 467: Warehouse Details");

        Container content = frame.getContentPane();
        content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
        
        //Menu bar with options
        menu = new JMenuBar();
        JMenu file = new JMenu("File");
        JMenuItem edit = new JMenuItem("Edit");
        JMenuItem save = new JMenuItem("Save");
        JMenuItem exec = new JMenuItem("Execute SQL");
        JMenuItem export = new JMenuItem("Export");
        
        //Give the menubar listeners
        edit.addActionListener(new MenuListener());
        save.addActionListener(new MenuListener2());
        exec.addActionListener(new MenuListener3());
        export.addActionListener(new MenuListener4());
        
        file.add(edit);
        file.add(save);
        file.add(exec);
        file.add(export);
        menu.add(file);
        
        content.add(menu);
        
        JPanel padding = new JPanel();
        content.add(padding);
        
        JPanel forTabs = new JPanel();
        
        JTabbedPane tabbed = new JTabbedPane();
        
        //Begin Overview pane
        JPanel overview = new JPanel();
        
        //Retrieve data set tables
        String[] tabs = DBConnect.getTableNames();
        String view = null;
        
        view = "Dataset "+MainWindow.selectedValue+"\n";
        view += "     Tables: \n";
        
        //Retrieve all columns
        for(int i = 0; i < tabs.length; i++) {
        	if(tabs[i] != null) {
        		view += "\t"+tabs[i]+"\n";

	        	Column[] tcolumns = DBConnect.getColumns(tabs[i]);
	        	view += "\tColumns:\n";
	        	for(int j = 0; j < tcolumns.length; j++) {
	        		view += "\t\t"+tcolumns[j].getName()+"\n";
	        	}
	        	view += "\n";
        	}
        }
        
        //Retrieve all warehouse information
        view += "Current Warehouse\n";
        view += "     Tables: \n";
        for(int i = 0; i < DBConnect.tindex; i++) {
        	view += "\t"+DBConnect.refNames[i]+"\n";
        	
        	Column[] cols = DBConnect.getColumns(DBConnect.refNames[i]);
			view += "\tColumns:\n";
        	for(int j = 0; j < cols.length; j++) {
        		if(cols[j] != null) {
        			view += "\t\t"+cols[j].getName()+"\n";
        		}
        	}
        }
        
		//Indpendent variables
        view += "\t"+SelectAttr.tablename.toUpperCase()+"\n";
        view += "\tColumns:\n";
		for(int i = 0; i < DBConnect.colnum; i++) {
			if(DBConnect.used[i] != null) {
				view += "\t\t"+DBConnect.used[i].getName()+"\n";
			}
		}
    	
		//Dpendent variables
		for(int i = 0; i < SelectDependents.full.length; i++) {
			view += "\t\t"+SelectDependents.full[i]+"\n";
		}        
		
		//Add "Overview" pane
        JTextArea over = new JTextArea(view,50,50);
        JScrollPane pane = new JScrollPane(over);
        pane.setPreferredSize(new Dimension(600, 700));

        overview.add(pane);
        
        //Retrieve code for tables besides main table
        for(int i = 0; i < DBConnect.refNames.length; i++) {
        	if(DBConnect.refNames[i] != null) {
	        	Column[] tcolumns = DBConnect.getColumns(DBConnect.refNames[i]);
	        	
	        	sql += "\n";
	        	sql += DBConnect.fillTable(DBConnect.refNames[i], tcolumns);
	        	sql += "\n";
        	}
        }
        
        //Create main header code
        sql += DBConnect.starTable();
        
		code = sql;
        
		//Add "Code" pane
        JPanel code = new JPanel();
        JTextArea text = new JTextArea(sql,50,50);
        JScrollPane pane2 = new JScrollPane(text);
        pane2.setPreferredSize(new Dimension(600, 700));

        code.add(pane2);
        
        //Diagram
        JPanel diagram = new JPanel();
        diagram.setPreferredSize(new Dimension(650, 750));
        
        Column[] cols = null;
        String info = null;
        boolean filled = false;
        boolean used[] = new boolean[10];
        used[4] = true;
        
        //If a filled, draw table in slot
        for(int i = 0; i < 9; i++) {
        	JPanel test = new JPanel();
        	JPanel test2 = new JPanel();
        	JLabel name = null;
        	JLabel hm = null;
        	
        	//If more tables exist
        	if(i < DBConnect.tindex) {
        		used[i] = true;
        		//If main table
        		if(i == 4) {   		
        			filled = true;
        			name = new JLabel(SelectAttr.tablename);
        			test.add(name);
                            		
        			//Display main table independent vars
        			for(int j = 0; j < DBConnect.colnum; j++) {
        				if(DBConnect.used[j] != null) {
        					info = DBConnect.used[j].getName()+" "+DBConnect.used[j].getType()+"(";
        					if(DBConnect.used[j].getPrec() == 0) {
        						info += DBConnect.used[j].getLength()+")";
        					}
        					else {
        						info += DBConnect.used[j].getPrec(); 
        						
        						if(DBConnect.used[j].getScale() != 0) {
        							info += ","+DBConnect.used[j].getScale();
        						}
        						info += ")";
        					}

        					hm = new JLabel(info);
        				}
        			}
					hm = new JLabel(info);
                    test2.add(hm);
	            	test.add(test2);
	            	
        			//Display main table dependent vars
	        		for(int j = 0; j < SelectDependents.vindex; j++) {
	        			info = " "+SelectDependents.vars[j].getAlias()+" "+SelectDependents.vars[j].getColumn().getType();
	        			
	        			if(SelectDependents.vars[j].getColumn().getPrec() == 0) {
	        				info += "("+SelectDependents.vars[j].getColumn().getLength()+"), \n";
	        			}
	        			else {
	        				info += "("+SelectDependents.vars[j].getColumn().getPrec(); 
	        				
	        				if(SelectDependents.vars[j].getColumn().getScale() != 0) {
	        					info += ","+SelectDependents.vars[j].getColumn().getScale();
	        				}
	        			    info += ") \n";
	        			}
						hm = new JLabel(info);
	                    test2.add(hm);
		            	test.add(test2);
	        		}
        		}
        		//If not main table
        		else {
        			filled = true;
        			name = new JLabel(DBConnect.refNames[i]);
        			test.add(name);
                    
        			cols = DBConnect.getColumns(DBConnect.refNames[i]);
        			
        			//Display all columns 
        			for(int j = 0; j < cols.length; j++) {
        				if(cols[j] != null) {
        					info = cols[j].getName()+" "+cols[j].getType()+"(";
        					if(cols[j].getPrec() == 0) {
        						info += cols[j].getLength()+")";
        					}
        					else {
        						info += cols[j].getPrec(); 
        						
        						if(cols[j].getScale() != 0) {
        							info += ","+cols[j].getScale();
        						}
        						info += ")";
        					}

        					hm = new JLabel(info);
                            test2.add(hm);
        	            	test.add(test2);
        				}
        			}
        		}
        	}
        	else {
        		//Display main table
        		if(i == 4) {
        			filled = true;
        			name = new JLabel(SelectAttr.tablename);
        			test.add(name);
        			
        			//Display main table independent vars
        			for(int j = 0; j < DBConnect.colnum; j++) {
        				if(DBConnect.used[j] != null) {
        					info = DBConnect.used[j].getName()+" "+DBConnect.used[j].getType()+"(";
        					if(DBConnect.used[j].getPrec() == 0) {
        						info += DBConnect.used[j].getLength()+")";
        					}
        					else {
        						info += DBConnect.used[j].getPrec(); 
        						
        						if(DBConnect.used[j].getScale() != 0) {
        							info += ","+DBConnect.used[j].getScale();
        						}
        						info += ")";
        					}

        					hm = new JLabel(info);
                            test2.add(hm);
                        	test.add(test2);
        				}
        			}
        			
        			//Display main table dependent vars
	        		for(int j = 0; j < SelectDependents.vindex; j++) {
	        			info = " "+SelectDependents.vars[j].getAlias()+" "+SelectDependents.vars[j].getColumn().getType();
	        			
	        			if(SelectDependents.vars[j].getColumn().getPrec() == 0) {
	        				info += "("+SelectDependents.vars[j].getColumn().getLength()+"), \n";
	        			}
	        			else {
	        				info += "("+SelectDependents.vars[j].getColumn().getPrec(); 
	        				
	        				if(SelectDependents.vars[j].getColumn().getScale() != 0) {
	        					info += ","+SelectDependents.vars[j].getColumn().getScale();
	        				}
	        			    info += ") \n";
	        			}
						hm = new JLabel(info);
	                    test2.add(hm);
		            	test.add(test2);
	        		}
        		}
        		//Not main table
        		else {
        			hm = new JLabel("");
                    test.add(hm);
        		}
        	}
        	
        	//If a table was filled, show it
        	if(filled) {
        		test2.setPreferredSize(new Dimension(170, 140));
                test.setPreferredSize(new Dimension(180, 180));
                test.setBackground(Color.WHITE);
            	test.setBorder(BorderFactory.createLineBorder(Color.black));     		
        	} 
        	else {
                test.setPreferredSize(new Dimension(180, 180));
        	}
        	
        	filled = false;
            diagram.add(test);     
            
            //Draw Lines connecting main table to any in positions 0-2
            if(i == 2) {
            	JPanel connect = new JPanel();
            	JLabel lines = null;
            	connect.setPreferredSize(new Dimension (800, 20));
            	
            	if(used[2]) {
            		lines = new JLabel("    \\                       |                         /");
                	connect.add(lines);
                	diagram.add(connect);
            	}
            	else if(used[1]) {
            		lines = new JLabel("    \\                       |                          ");
                	connect.add(lines);
                	diagram.add(connect);
            	}
            	else if(used[0]) {
            		lines = new JLabel("    \\                                                  ");
                	connect.add(lines);
                	diagram.add(connect);
            	}
            	else {
                	diagram.add(connect);
            	}
            }
            else if(i == 3) {
            	JPanel connect = new JPanel();
            	JPanel pad = new JPanel();
            	JLabel lines = new JLabel("_");
            	pad.setPreferredSize(new Dimension(15,0));
            	connect.setPreferredSize(new Dimension (15, 120));
            	
            	connect.add(pad);

            	if(used[3]) {
            		connect.add(lines);
                	diagram.add(connect);
            	}
            }
            else if(i == 4) {
                //Draw Lines connecting main table to any in positions 4 and 5
            	JPanel connect = new JPanel();
            	JPanel pad = new JPanel();
            	JLabel lines = new JLabel("_");
            	pad.setPreferredSize(new Dimension(15,0));
            	connect.setPreferredSize(new Dimension (15, 120));
            	
            	connect.add(pad);

            	if(DBConnect.tindex > 5) {
            		connect.add(lines);
                	diagram.add(connect);
            	}
            }
            //Draw Lines connecting main table to any in positions 7-9
            else if(i == 5) {
            	JPanel connect = new JPanel();
            	JLabel lines = null;
            	connect.setPreferredSize(new Dimension (800, 20));
            	
            	if(DBConnect.tindex > 7) {
            		lines = new JLabel("  /                       |                     \\");
                	connect.add(lines);
                	diagram.add(connect);
            	}
            	else if(DBConnect.tindex > 6) {
            		lines = new JLabel("  /                       |                       ");
                	connect.add(lines);
                	diagram.add(connect);
            	}
            	else if(DBConnect.tindex > 5) {
            		lines = new JLabel("  /                                               ");
                	connect.add(lines);
                	diagram.add(connect);
            	}
            	else {
            		diagram.add(connect);
            	}
            }
        }
        
        //Add all three panes
        tabbed.addTab("Overview", overview);
        tabbed.addTab("SQL", code);
        tabbed.addTab("Diagram", diagram);
        
        forTabs.add(tabbed);
        content.add(forTabs);
  
        //Display the window.
        frame.setLocationRelativeTo(SelectDependents.frame);
        frame.setPreferredSize(new Dimension(700, 800));
        frame.pack();
        frame.setVisible(true); 
	}
	
	/**
	 * Creates a menu listener for the "Back" item.
	 */
    static class MenuListener implements ActionListener {
       	/**
    	 * Used to catch when the back item is selected.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		SelectDependents.createSelectDependents();
    		frame.setVisible(false);
    	}
    }
    
	/**
	 * Creates a menu listener for the "Save" item.
	 */
    static class MenuListener2 implements ActionListener {
       	/**
    	 * Used to catch when the save item is selected.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		ChooseFile.saveState();
    	}
    }
    
	/**
	 * Creates a menu listener for the "Execute SQL" item.
	 */
    static class MenuListener3 implements ActionListener {
       	/**
    	 * Used to catch when the execute sql item is selected.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		DBConnect.execute(code);
    	}
    }
 
	/**
	 * Creates a menu listener for the "Export SQL" item.
	 */
    static class MenuListener4 implements ActionListener {
       	/**
    	 * Used to catch when the export sql item is selected.
    	 * 
    	 * @param e		Mouse pressed action event
    	 */
    	public void actionPerformed(ActionEvent e) {
    		ExportSQLGUI.chooseNameGUI();
    	}
    }
}
