package seniorproj;

/**
 * Creates a tablekey object containing information regarding the table name, 
 * and information pertaining to all of its foreign keys.
 * 
 * @author Steven Weigand
 *
 */
public class TableKeys {
	
	String tname;
	String tabkeys[];
	String tabpkeys[];
	String colkeys[];
	boolean visited;
	
	/**
	 * Default TableKeys constructor for initialization.
	 */
	public TableKeys() {}
	
	/**
	 * Creates and initializes a TableKey object.
	 * 
	 * @param tname		Name of the table
	 * @param tabkeys	The FKTABLE_NAME array info
	 * @param colkeys	The FKCOLUMN_NAME array info
	 * @param tabpkeys	The PKTABLE_NAME array info
	 */
	public TableKeys(String tname, String tabkeys[], String colkeys[], String tabpkeys[]) {
		this.tname = tname;
		this.tabkeys = tabkeys;
		this.colkeys = colkeys;
		this.tabpkeys = tabpkeys;
		this.visited = false;
	}
	
	/**
	 * Retrieve the name.
	 * 
	 * @return		String containing table name
	 */
	public String getName() {
		return tname;
	}
	
	/**
	 * Retrieve the foreign key table names.
	 * 
	 * @return		String[] containing foreign key table names
	 */
	public String[] getTabs() {
		return tabkeys;
	}
	
	/**
	 * Retrieve the foreign key column names.
	 * 
	 * @return		String[] containing foreign key column names
	 */
	public String[] getCols() {
		return colkeys;
	}
	
	/**
	 * Retrieve the primary key table names.
	 * 
	 * @return		String[] containing primary key table names
	 */
	public String[] getPTabs() {
		return tabpkeys;
	}
}
